import yaml
import warnings

from nltk import NaiveBayesClassifier, MaxentClassifier
from nltk.classify.util import accuracy

with warnings.catch_warnings():
    warnings.simplefilter("ignore")
    from nltk.sentiment import SentimentAnalyzer
    from nltk.sentiment.util import extract_unigram_feats, extract_bigram_feats
from nltk.corpus import stopwords


def remove_collection_from_string(text, collection):
    for string in collection:
        text = text.replace(string, '')
    return text


with open('reviews.yaml', encoding='utf8') as f:
    reviews = yaml.load(f)

analyzer = SentimentAnalyzer()

all_names = [name for company in reviews.keys() for name in reviews[company]['names']]
all_names.sort(key=lambda x: len(x.split()), reverse=True)

texts = []
all_words = []

all_docs = []

all_reviews = []

for company in reviews.keys():
    for review in reviews[company]['reviews']:
        # remove company name
        text = remove_collection_from_string(review['text'].lower(), ".,:;?!+\'\"/\\(){}[]<>")
        text = remove_collection_from_string(text, all_names)
        text = remove_collection_from_string(text, '-')
        text = list(filter(lambda x: x not in stopwords.words('norwegian'), text.split()))

        texts.append(text)
        all_words.extend(text)

        all_docs.append((review['text'].split(), review['polarity']))

unigrams = analyzer.unigram_word_feats(all_words, min_freq=10)
bigrams = analyzer.bigram_collocation_feats(texts, min_freq=10)

analyzer.add_feat_extractor(extract_unigram_feats, unigrams=unigrams)
analyzer.add_feat_extractor(extract_bigram_feats, bigrams=bigrams)

training_set = all_docs[:int(len(all_docs) * 0.5)]
test_set = all_docs[int(len(all_docs) * 0.5):]

training_set = analyzer.apply_features(training_set)
test_set = analyzer.apply_features(test_set)

trainer = MaxentClassifier.train


analyzer.train(trainer, training_set)
print(accuracy(analyzer.classifier, test_set))

analyzer.classifier.show_most_informative_features(80)
