def compileNorskOrdvev():
    words_file = 'LexicalResources/Norsk ordvev 1.1.2/dat/words.tab'
    wordsences_file = 'LexicalResources/Norsk ordvev 1.1.2/dat/wordsenses.tab'
    target_file = 'LexicalResources/compiledNorskOrdvev-1.1.2.tab'

    words = {}
    with open(words_file, encoding='utf8') as f:
        for line in f:
            if line.strip() == '':
                continue
            spl = line.split()
            words[spl[0]] = (' '.join(spl[1:-1]), spl[-1])

    # print(set([i[1] for i in words.values()]))
    synsets = {}
    with open(wordsences_file, encoding='utf8') as f:
        for line in f:
            if line.strip() == '':
                continue
            wordsence_id, word_id, synset_id, *register = line.split()
            if synset_id not in synsets:
                synsets[synset_id] = []
            synsets[synset_id].append(word_id)

    to_remove = []
    #translate word_ids to words
    for synset_id in synsets.keys():
        # eliminate duplicates
        synsets[synset_id] = list(set(synsets[synset_id]))
        for i in range(len(synsets[synset_id])):
            if synsets[synset_id][i] in words:
                synsets[synset_id][i] = words[synsets[synset_id][i]]
            else:
                synsets[synset_id][i] = ''
        while '' in synsets[synset_id]:
            synsets[synset_id].remove('')
        if len(synsets[synset_id]) == 0:
            to_remove.append(synset_id)
    for id in to_remove:
        del synsets[id]

    # # see synets with different POS
    # for value in synsets.values():
    #     if len(value) > 1:
    #         forste = value[0][1]
    #         for pos in value[1:]:
    #             if pos[1] != forste:
    #                 print(value)

    with open(target_file, 'w', encoding='utf8') as f:
        for synset_id, words in sorted(synsets.items(), key=lambda x: len(x[1]), reverse=True):
            line = '{}\t'.format(synset_id) + '\t'.join(['{}#{}'.format(word,pos) for word,pos in words]) + '\n'
            f.write(line)

norsk_ordvev_to_obt_pos_mapping = {
    'Noun': 'subst',
    'Verb': 'verb',
    'Adjective': 'adj',
}


def read_compiled_norsk_ordvev(key='lemma,pos', eliminateNone=True):
    if key not in ('lemma,pos', 'id'):
        raise AttributeError('key='+key, 'is not allowed')
    norsk_ordvev = {}
    with open('LexicalResources/compiledNorskOrdvev-1.1.2.tab', encoding='utf8') as f:
        if key == 'id':
            for line in f:
                id, *lemmapos = line.split()
                if id not in norsk_ordvev.keys():
                    norsk_ordvev[id] = []
                lemmapos = iter(lemmapos)
                for spl in lemmapos:
                    while '#' not in spl:
                        spl = spl + ' ' + next(lemmapos)
                    lemma, pos = spl.split('#')
                    if pos not in norsk_ordvev_to_obt_pos_mapping.keys() and eliminateNone:
                        continue
                    norsk_ordvev[id].append((lemma, norsk_ordvev_to_obt_pos_mapping.get(pos, pos)))

        elif key == 'lemma,pos':
            for line in f:
                id, *lemmalist = line.split()
                lemmalist = iter(lemmalist)
                for spl in lemmalist:
                    while '#' not in spl:
                        spl = spl + ' ' + next(lemmalist)
                    lemma, pos = spl.split('#')
                    if pos not in norsk_ordvev_to_obt_pos_mapping.keys() and eliminateNone:
                        continue
                    lemmapos = (lemma, norsk_ordvev_to_obt_pos_mapping.get(pos, pos))
                    if lemmapos not in norsk_ordvev.keys():
                        norsk_ordvev[lemmapos] = []
                    norsk_ordvev[lemmapos].append(id)

    return norsk_ordvev



if __name__ == '__main__':
    compileNorskOrdvev()