def load_norwegian_wordnet(lang='nob'):
    nord_net = {}
    with open('LexicalResources/NorNet/wn-data-{}.tab'.format(lang), encoding='utf8') as f:
        for line in f:
            if line.strip()[0] == '#' or len(line.split('\t')) != 3:
                # commented or wrong format
                continue

            identifier, wn_type, word = line.split('\t')
            synset_id, pos = identifier.split('-')
            key = (word.strip(), pos)
            if key not in nord_net:
                nord_net[key] = set()
            nord_net[key].add(synset_id)
    with open('LexicalResources/NorNet/wn-wikt-{}.tab'.format(lang), encoding='utf8') as f:
        for line in f:
            if line.strip()[0] == '#' or len(line.split()) != 3:
                # commented or wrong format
                continue

            identifier, wn_type, word = line.split('\t')
            synset_id, pos = identifier.split('-')
            key = (word, pos)
            if key not in nord_net:
                nord_net[key] = set()
            nord_net[key].add(synset_id)

    return nord_net


if __name__ == '__main__':
    import time

    print('Start loading Norwegian Wordnet by Lars Nygaard')
    start = time.time()
    nord_net = load_norwegian_wordnet()
    print('Length', len(nord_net))
    print('Finished loading in', time.time() - start)
