import time
import re

from TextClasses import SentiWordNetEntry
from threading import Lock

regex = re.compile(r"#([1-9][0-9]*)$", re.IGNORECASE)
underscore = re.compile(r"_")

sentiWordNet = set()
sentiLock = Lock()


def loadWithIdPosKey(file):
    if not len(sentiWordNet):
        readSentiWordNet(file)
    # check for lock
    sentiLock.acquire() and sentiLock.release()
    largeList = {}
    for sentiObject in sentiWordNet:
        largeList[(sentiObject.sentiwordnet_id, sentiObject.part_of_speech)] = sentiObject
    return largeList


def loadWithLemmaKey(file):
    if not len(sentiWordNet):
        readSentiWordNet(file)
    # check for lock
    sentiLock.acquire() and sentiLock.release()
    largeList = {}
    for sentiObject in sentiWordNet:
        for lemma, sense in sentiObject.lemmas:
            if lemma not in largeList.keys():
                largeList[lemma] = []
            largeList[lemma].append(sentiObject)
    return largeList


# acquire sentiLock before calling this method
def readSentiWordNet(filename):
    with open(filename) as f:
        for line in f:
            if line.strip()[0] == '#':
                continue
            sentiWordNet.add(createSentiObject(line))


def createSentiObject(line) -> SentiWordNetEntry:
    pos, id, pos_score, neg_score, *rest = line.split()
    lemmalist = []
    explaination = ''
    for word in rest:
        if explaination or not regex.search(word):
            explaination += word + ' '
            continue
        word, sense_number = word.split('#')
        word = underscore.sub(' ', word)
        lemmalist.append((word, sense_number))
    return SentiWordNetEntry(id, pos, pos_score, neg_score, lemmalist, explaination)


if __name__ == '__main__':
    start = time.time()
    loadWithIdPosKey('SentiWordNet_3.0.0_20130122.txt')
    print('SentiWordNet loaded in', time.time() - start, 'seconds')

    start = time.time()
    loadWithLemmaKey('SentiWordNet_3.0.0_20130122.txt')
    print('SentiWordNet loaded in', time.time() - start, 'seconds')
