import yaml
from readNorskOrdvev import read_compiled_norsk_ordvev


def count_reviews():
    with open('reviews.yaml') as f:
        reviews = yaml.load(f)
    print('Totalt:', sum([len(company['reviews']) for company in reviews.values()]))
    print('Positive', sum([len(list(filter(lambda x: x['polarity'] == 1, liste['reviews']))) for company, liste in reviews.items()]), sep='\t')
    print('Nøytrale', sum([len(list(filter(lambda x: x['polarity'] == 0, liste['reviews']))) for company, liste in reviews.items()]), sep='\t')
    print('Negative', sum([len(list(filter(lambda x: x['polarity'] == -1, liste['reviews']))) for company, liste in reviews.items()]), sep='\t')
    print()
    for company, liste in reviews.items():
        print('Company', company, sep='\t')
        print('Positive', sum([len(list(filter(lambda x: x['polarity'] == 1, liste['reviews'])))]), sep='\t')
        print('Nøytrale', sum([len(list(filter(lambda x: x['polarity'] == 0, liste['reviews'])))]), sep='\t')
        print('Negative', sum([len(list(filter(lambda x: x['polarity'] == -1, liste['reviews'])))]), sep='\t')
        # print('Objektiv:',
        #       sum([len(list(filter(lambda x: x['subj'] is False, company['reviews']))) for company in reviews.values()]))


def readRelations():
    ordvev = read_compiled_norsk_ordvev('id')
    relations = set()
    with open('Norsk ordvev 1.1.2/dat/relations.tab') as f:
        for line in f:
            if line.strip().startswith('#'):
                continue
            line = line.split()
            lemmaposlist = ordvev.get(line[0], [])
            if len(lemmaposlist) == 1 and lemmaposlist[0][1] == 'n' and lemmaposlist[0][0][0].isupper():
                hit = ordvev.get(line[3], [])
                if len(hit) > 0 and not (hit[0][1] == 'n' and hit[0][0][0].isupper()):
                    relations.add((line[1], tuple(hit)))
    for i in relations:
        print(i)


def count_distinct_wn_ids():
    ids = set()
    with open('LexicalResources/NorNet/wn-data-nob.tab', encoding='utf8') as f:
        for line in f:
            if not line.strip() or line.strip().startswith('#'):
                continue
            line = line.split()
            ids.add(line[-1])
    with open('LexicalResources/NorNet/wn-wikt-nob.tab', encoding='utf8') as f:
        for line in f:
            if not line.strip() or line.strip().startswith('#'):
                continue
            line = line.split()
            ids.add(line[-1])
    print(len(ids))


def add_subjective_field():
    with open('reviews.yaml', encoding='utf8') as f:
        reviews = yaml.load(f)
    for company in reviews.keys():
        for obj in reviews[company]['reviews']:
            obj['subj'] = None

    with open('reviews.yaml', 'w', encoding='utf8') as f:
        yaml.dump(reviews, f, encoding='utf-8', allow_unicode=True, default_flow_style=False, default_style='')


def set_subjective():
    with open('reviews.yaml', encoding='utf8') as f:
        reviews = yaml.load(f)
    mapping = {'s': True, 'o': False}
    for company in reviews.keys():
        for review in reviews[company]['reviews']:
            if review['subj'] is not None:
                continue
            print(review['text'])
            i = input('(s)ubjektiv/(o)bjektiv? ')
            if i == 'exit':
                with open('reviews.yaml', 'w', encoding='utf8') as f:
                    yaml.dump(reviews, f, encoding='utf-8', allow_unicode=True, default_flow_style=False,
                              default_style='')
            while i not in ('s', 'o'):
                i = input('subjektiv/objektiv? ')
            review['subj'] = mapping[i]

    with open('reviews.yaml', 'w', encoding='utf8') as f:
        yaml.dump(reviews, f, encoding='utf-8', allow_unicode=True, default_flow_style=False, default_style='')


def list_review_count():
    with open('reviews.yaml', encoding='utf8') as f:
        reviews = yaml.load(f)

    ob = 0
    sub = 0
    for comp in reviews.keys():
        ob += len(list(filter(lambda x: not x['subj'], reviews[comp]['reviews'])))
        sub += len(list(filter(lambda x: x['subj'], reviews[comp]['reviews'])))

    print('Objective:', ob, '\nSubjective:', sub)


def list_pos_in_fullform_bm():
    i = set()
    with open('LexicalResources/Norsk ordvev 1.1.2/dat/synset_attributes.tab', encoding='utf8') as f:
        for line in f:
            if line.split('\t')[2] == 'ban\n':
                i.add(line.split('\t')[0])
    import readNorskOrdvev
    no = readNorskOrdvev.read_compiled_norsk_ordvev('id')
    i = list(i)
    for index in range(len(i)):
        if i[index] in no.keys():
            print(no[i[index]])


def remove_dup():
    with open('LexicalResources/custom_bank_lexicon.yaml', encoding='utf8') as f:
        words = list(set(yaml.load(f)['words']))
    with open('LexicalResources/custom_bank_lexicon.yaml', 'w', encoding='utf8') as f:
        yaml.dump({'words': words}, f, encoding='utf-8', allow_unicode=True, default_flow_style=False, default_style='')

def count_pos_in_pol_cat():
    import obt_tagger
    import helpers
    with open('reviews.yaml', encoding='utf8') as f:
        reviews = yaml.load(f)
    negative_reviews = [review['text'] for review_comps in reviews.values() for review in review_comps['reviews'] if review['polarity'] == -1]
    neutral_reviews =  [review['text'] for review_comps in reviews.values() for review in review_comps['reviews'] if review['polarity'] == 0]
    positive_reviews = [review['text'] for review_comps in reviews.values() for review in review_comps['reviews'] if review['polarity'] == 1]
    negative_reviews_pos_stats = {
        'adj': 0, 'adv': 0, 'det': 0, 'inf-merke': 0,
        'interj': 0, 'konj': 0, 'prep': 0, 'pron': 0,
        'sbu': 0, 'subst': 0, 'ukjent': 0, 'verb': 0
    }
    neutral_reviews_pos_stats = {
        'adj': 0, 'adv': 0, 'det': 0, 'inf-merke': 0,
        'interj': 0, 'konj': 0, 'prep': 0, 'pron': 0,
        'sbu': 0, 'subst': 0, 'ukjent': 0, 'verb': 0
    }
    positive_reviews_pos_stats = {
        'adj': 0, 'adv': 0, 'det': 0, 'inf-merke': 0,
        'interj': 0, 'konj': 0, 'prep': 0, 'pron': 0,
        'sbu': 0, 'subst': 0, 'ukjent': 0, 'verb': 0
    }

    OBTTAgger = obt_tagger.OBTagger('../The-Oslo-Bergen-Tagger')
    for text in negative_reviews:
        for obt_result in helpers.parse_obt_results(OBTTAgger.run(text), {}):
            for word in obt_result.list:
                if word.pos in negative_reviews_pos_stats.keys():
                    negative_reviews_pos_stats[word.pos] = negative_reviews_pos_stats[word.pos] + 1
    for text in neutral_reviews:
        for obt_result in helpers.parse_obt_results(OBTTAgger.run(text), {}):
            for word in obt_result.list:
                if word.pos in neutral_reviews_pos_stats.keys():
                    neutral_reviews_pos_stats[word.pos] = neutral_reviews_pos_stats[word.pos] + 1
    for text in positive_reviews:
        for obt_result in helpers.parse_obt_results(OBTTAgger.run(text), {}):
            for word in obt_result.list:
                if word.pos in positive_reviews_pos_stats.keys():
                    positive_reviews_pos_stats[word.pos] = positive_reviews_pos_stats[word.pos] + 1

    print('Negative review pos')
    for pos, freq in negative_reviews_pos_stats.items():
        print(str(pos)+','+str(freq))
    print('Neutral review pos')
    for pos, freq in neutral_reviews_pos_stats.items():
        print(str(pos)+','+str(freq))
    print('Positive review pos')
    for pos, freq in positive_reviews_pos_stats.items():
        print(str(pos)+','+str(freq))


def convert_to_sentence():
    import nltk
    nor_tokenizer = nltk.data.load('tokenizers/punkt/norwegian.pickle')
    with open('reviews.yaml', encoding='utf8') as f:
        reviews = yaml.load(f)

    for company, value in reviews.items():
        for review in value['reviews']:
            review['documentPolarity'] = review['polarity']
            review['documentSubjective'] = review['subj']
            del review['polarity']
            del review['subj']
            review['sentences'] = []
            sentences = nor_tokenizer.tokenize(review['text'])
            for sent in sentences:
                review['sentences'].append({})
                review['sentences'][-1]['text'] = sent
                review['sentences'][-1]['polarity'] = review['documentPolarity']
                review['sentences'][-1]['subjective'] = review['documentSubjective']
            del review['text']
            print(review)

    with open('reviews.yaml', 'w', encoding='utf8') as f:
        yaml.dump(reviews, f, encoding='utf-8', allow_unicode=True, default_flow_style=False, default_style='')

def count_bytt_no():
    with open('reviews.yaml', encoding='utf8') as f:
        reviews = yaml.load(f)

    antall = 0
    for comp in reviews.keys():
        for review in reviews[comp]['reviews']:
            if review['origin'] and review['origin'].startswith('https://www.bytt.no'):
                antall += 1
    print(antall)

def evalutate_custom_swn_mapping():
    import readNorwegianWordnet, norwegianPWNMapping, readSentiWordNet
    norwegian_wordnet = readNorwegianWordnet.load_norwegian_wordnet('nob')
    norsk_ordvev = read_compiled_norsk_ordvev()
    pwn_mapping = norwegianPWNMapping.readCompiledNoToPWN()
    swn = readSentiWordNet.loadWithIdPosKey('LexicalResources/SentiWordNet_3.0.0_20130122.txt')

    swn_to_obt_mapping = {
        'n': 'subst',
        'v': 'verb',
        'a': 'adj',
        'r': 'adv'
    }
    accuracy = []
    avg_sence_numbers_nwordnet = []
    avg_sence_numbers_pwn_mapping = []

    for (word, pos), synset_ids in norwegian_wordnet.items():
        avg_sence_numbers_nwordnet.extend([int(sence) for synset_id in synset_ids for lemma, sence in swn[(synset_id, pos)].lemmas])

        nor_synsets = norsk_ordvev.get((word, swn_to_obt_mapping.get(pos, None)), set())

        if not nor_synsets:
            continue

        # map to swn
        temp = []
        i = 0
        while i < len(nor_synsets):
            pwn_map = pwn_mapping.get(nor_synsets[i], None)
            if pwn_map:
                nor_synsets[i] = pwn_map[0]
                temp.append(pwn_map[1])
                i += 1
            else:
                nor_synsets.pop(i)
        if not len(nor_synsets):
            continue

        avg_sence_numbers_pwn_mapping.extend([int(sence) for i in range(len(nor_synsets)) for lemma, sence in swn[(nor_synsets[i], temp[i])].lemmas])

        accuracy.append(2.0 * len(set(nor_synsets).intersection(synset_ids)) / (len(synset_ids) + len(nor_synsets)))
    print(float(sum(avg_sence_numbers_nwordnet)) / len(avg_sence_numbers_nwordnet))
    print(float(sum(avg_sence_numbers_pwn_mapping)) / len(avg_sence_numbers_pwn_mapping))
    print(sum(accuracy) / len(accuracy))

def inter_annotator_agreement():
    #from krippendorff_alpha import krippendorff_alpha, nominal_metric
    with open('reviews.yaml', encoding='utf8') as f:
        original = yaml.load(f)
    with open('reviews_nicole.yaml', encoding='utf8') as f:
        annotated = yaml.load(f)

    companies = original.keys()
    a = 0
    anders_annotations = {}
    nicole_annotations = {}
    for comp in companies:
        for i in range(len(original[comp]['reviews'])):
            a += 1
            anders_annotations[str(i) + '_pol'] = original[comp]['reviews'][i]['polarity']
            anders_annotations[str(i) + '_subj'] = original[comp]['reviews'][i]['subj']
            nicole_annotations[str(i) + '_pol'] = annotated[comp]['reviews'][i]['annotations']['nicole']['polarity']
            nicole_annotations[str(i) + '_subj'] = annotated[comp]['reviews'][i]['annotations']['nicole']['subjective']

    data = [anders_annotations, nicole_annotations]
    #print(krippendorff_alpha(data,  missing_items='*'))#, metric=nominal_metric))

if __name__ == '__main__':
    count_reviews()
